#include <stdio.h>
#include <linux/kernel.h>
#include <linux/sys.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>

// files to write out
#define LOAD_FILE	"/var/status/load.history"
#define MEM_FILE	"/var/status/mem.history"
#define SWAP_FILE	"/var/status/swap.history"

#define PID_FILE	"/var/run/trackstatus.pid"

// last 61 (!) mins of history for these quantities
float load[61];
float mem[61];
float swap[61];

void sighandler(int num);
void gatherinfo();

void sighandler(int num)
{
	int i;
	FILE *fp;

	if (num == SIGUSR1 || num == SIGHUP)
	{
		// dump history out to files
		fp = fopen(LOAD_FILE, "wt");
	
		for (i = 0; i < 61; i++)
			fprintf(fp, "%f\n", load[i]);
	
		fclose(fp);
	
		fp = fopen(MEM_FILE, "wt");
	
		for (i = 0; i < 61; i++)
			fprintf(fp, "%f\n", mem[i]);
	
		fclose(fp);

		fp = fopen(SWAP_FILE, "wt");
	
		for (i = 0; i < 61; i++)
			fprintf(fp, "%f\n", swap[i]);

		fclose(fp);

		return;
	}
	else if (num == SIGALRM)
	{
		gatherinfo();

		// reset alarm
		alarm(60);

		return;
	}
	else
	{
		unlink(PID_FILE);
		_exit(0);
	}
}

void gatherinfo()
{
	struct sysinfo info;
	int i;

	for (i = 0; i < 60; i++)
	{
		load[i] = load[i + 1];
		mem[i]  =  mem[i + 1];
		swap[i] = swap[i + 1];
	}

	sysinfo(&info);

	load[60] = (float)info.loads[0] / 64000.0f; // magic number ;(
	mem[60]  = (float)(info.totalram - info.freeram)
				/ (float)info.totalram;
	swap[60] = (float)(info.totalswap - info.freeswap)
				/ (float)info.totalswap;
}

int main()
{
	pid_t pid;
	FILE *fp;
	int i;
	// bunch of signals we catch so that we can remove our pid file
	int signals[] = {SIGHUP, SIGINT, SIGQUIT, SIGILL, SIGABRT, 
			 SIGFPE, SIGSEGV, SIGPIPE, SIGALRM, SIGTERM,
			 SIGUSR1, SIGUSR2, SIGTSTP, SIGTTIN, SIGTTOU};

	// fork to put ourselves in the background
	if (fork())
		return 0;

	// write out the pid
	pid = getpid();

	fp = fopen(PID_FILE, "wt");

	if (!fp)
	{
		perror("trackstatus: fopen");
		return 1;
	}

	fprintf(fp, "%d", pid);
	fclose(fp);

	// set the signal handler
	for (i = 0; i < sizeof(signals) / sizeof(int); i++)
		__bsd_signal(signals[i], sighandler);

	alarm(1);

	// go into a loop
	for (;;)
	{
		// how long we sleep for doesn't matter, since we'll probably
		// get interrupted by a signal anyway
		sleep(60);
	}
}
